@@ -17,7 +17,7 @@
  */
 
 /*
- * $Id: linkfd.c,v 1.4.2.6 2001/01/10 01:46:29 maxk Exp $
+ * linkfd.c,v 1.4.2.15.2.2 2006/11/16 04:03:23 mtbishop Exp
  */
 
 #include "config.h"
@@ -50,6 +50,9 @@
 #include "lib.h"
 #include "driver.h"
 
+/* used by lfd_encrypt */
+int send_a_packet = 0;
+
 /* Host we are working with. 
  * Used by signal handlers that's why it is global. 
  */
@@ -156,12 +159,20 @@ inline int lfd_check_up(void)
 		
 /********** Linker *************/
 /* Termination flag */
-static int linker_term;
+static volatile sig_atomic_t linker_term;
 
 static void sig_term(int sig)
 {
-     syslog(LOG_INFO,"Closing connection");
-     linker_term = 1;
+     vtun_syslog(LOG_INFO, "Closing connection");
+     io_cancel();
+     linker_term = VTUN_SIG_TERM;
+}
+
+static void sig_hup(int sig)
+{
+     vtun_syslog(LOG_INFO, "Reestablishing connection");
+     io_cancel();
+     linker_term = VTUN_SIG_HUP;
 }
 
 /* Statistic dump */
@@ -194,10 +205,10 @@ int lfd_linker(void)
      struct timeval tv;
      char *buf, *out;
      fd_set fdset;
-     int maxfd, idle = 0;
+     int maxfd, idle = 0, tmplen;
 
      if( !(buf = lfd_alloc(VTUN_FRAME_SIZE + VTUN_FRAME_OVERHEAD)) ){
-	syslog(LOG_ERR,"Can't allocate buffer for the linker"); 
+	vtun_syslog(LOG_ERR,"Can't allocate buffer for the linker"); 
         return 0; 
      }
 
@@ -223,12 +234,33 @@ int lfd_linker(void)
 	   else
 	      continue;
 	} 
-	
+	if (send_a_packet)
+        {
+           send_a_packet = 0;
+           tmplen = 1;
+	   lfd_host->stat.byte_out += tmplen; 
+	   if( (tmplen=lfd_run_down(tmplen,buf,&out)) == -1 )
+	      break;
+	   if( tmplen && proto_write(fd1, out, tmplen) < 0 )
+	      break;
+	   lfd_host->stat.comp_out += tmplen; 
+        }
 	if( !len ){
+           if (send_a_packet)
+           {
+              send_a_packet = 0;
+              tmplen = 1;
+	      lfd_host->stat.byte_out += tmplen; 
+   	      if( (tmplen=lfd_run_down(tmplen,buf,&out)) == -1 )
+	         break;
+	      if( tmplen && proto_write(fd1, out, tmplen) < 0 )
+	         break;
+	      lfd_host->stat.comp_out += tmplen; 
+           }
 	   /* We are idle, lets check connection */
 	   if( lfd_host->flags & VTUN_KEEP_ALIVE ){
 	      if( ++idle > lfd_host->ka_failure ){
-	         syslog(LOG_INFO,"Session %s network timeout", lfd_host->host);
+	         vtun_syslog(LOG_INFO,"Session %s network timeout", lfd_host->host);
 		 break;	
 	      }
 	      /* Send ECHO request */
@@ -250,7 +282,7 @@ int lfd_linker(void)
            len = len & VTUN_FSIZE_MASK;
 	   if( fl ){
 	      if( fl==VTUN_BAD_FRAME ){
-		 syslog(LOG_ERR, "Received bad frame");
+		 vtun_syslog(LOG_ERR, "Received bad frame");
 		 continue;
 	      }
 	      if( fl==VTUN_ECHO_REQ ){
@@ -264,7 +296,7 @@ int lfd_linker(void)
 		 continue;
 	      }
 	      if( fl==VTUN_CONN_CLOSE ){
-	         syslog(LOG_INFO,"Connection closed by other side");
+	         vtun_syslog(LOG_INFO,"Connection closed by other side");
 		 break;
 	      }
 	   }   
@@ -301,8 +333,12 @@ int lfd_linker(void)
 	}
      }
      if( !linker_term && errno )
-	syslog(LOG_INFO,"%s (%d)", strerror(errno), errno);
-	
+	vtun_syslog(LOG_INFO,"%s (%d)", strerror(errno), errno);
+
+     if (linker_term == VTUN_SIG_TERM) {
+       lfd_host->persist = 0;
+     }
+
      /* Notify other end about our close */
      proto_write(fd1, buf, VTUN_CONN_CLOSE);
      lfd_free(buf);
@@ -313,7 +349,7 @@ int lfd_linker(void)
 /* Link remote and local file descriptors */ 
 int linkfd(struct vtun_host *host)
 {
-     struct sigaction sa, sat;
+     struct sigaction sa, sa_oldterm, sa_oldint, sa_oldhup;
      int old_prio;
 
      lfd_host = host;
@@ -339,7 +375,10 @@ int linkfd(struct vtun_host *host)
 
      memset(&sa, 0, sizeof(sa));
      sa.sa_handler=sig_term;
-     sigaction(SIGTERM,&sa,&sat);
+     sigaction(SIGTERM,&sa,&sa_oldterm);
+     sigaction(SIGINT,&sa,&sa_oldint);
+     sa.sa_handler=sig_hup;
+     sigaction(SIGHUP,&sa,&sa_oldhup);
 
      /* Initialize statstic dumps */
      if( host->flags & VTUN_STAT ){
@@ -355,19 +394,24 @@ int linkfd(struct vtun_host *host)
 	   setvbuf(host->stat.file, NULL, _IOLBF, 0);
 	   alarm(VTUN_STAT_IVAL);
 	} else
-	   syslog(LOG_ERR, "Can't open stats file %s", file);
+	   vtun_syslog(LOG_ERR, "Can't open stats file %s", file);
      }
 
+     io_init();
+
      lfd_linker();
 
      if( host->flags & VTUN_STAT ){
         alarm(0);
-	fclose(host->stat.file);
+	if (host->stat.file)
+	  fclose(host->stat.file);
      }
 
      lfd_free_mod();
      
-     sigaction(SIGTERM,&sat,NULL);
+     sigaction(SIGTERM,&sa_oldterm,NULL);
+     sigaction(SIGINT,&sa_oldint,NULL);
+     sigaction(SIGHUP,&sa_oldhup,NULL);
 
      setpriority(PRIO_PROCESS,0,old_prio);
 
