@@ -17,8 +17,9 @@
  */
 
 /*
- * $Id: main.c,v 1.1.1.2.2.3 2001/01/10 01:46:29 maxk Exp $
+ * main.c,v 1.1.1.2.2.8.2.4 2006/11/16 04:03:41 mtbishop Exp
  */ 
+
 #include "config.h"
 
 #include <stdio.h>
@@ -28,6 +29,7 @@
 #include <signal.h>
 #include <fcntl.h>
 #include <syslog.h>
+#include <sys/mman.h>
 
 #ifdef HAVE_NETINET_IN_H
 #include <netinet/in.h>
@@ -67,10 +69,13 @@ int main(int argc, char *argv[], char *env[])
      vtun.ifcfg = strdup("/sbin/ifconfig");
      vtun.route = strdup("/sbin/route");
      vtun.fwall = strdup("/sbin/ipchains");	
+     vtun.iproute = strdup("/sbin/ip");	
 
      vtun.svr_name = NULL;
-     vtun.svr_port = -1;
+     vtun.svr_addr = NULL;
+     vtun.bind_addr.port = -1;
      vtun.svr_type = -1;
+     vtun.syslog   = LOG_DAEMON;
 
      /* Initialize default host options */
      memset(&default_host, 0, sizeof(default_host));
@@ -79,19 +84,29 @@ int main(int argc, char *argv[], char *env[])
      default_host.timeout = VTUN_CONNECT_TIMEOUT;
      default_host.ka_interval = 30;
      default_host.ka_failure  = 4;
+     default_host.loc_fd = default_host.rmt_fd = -1;
 
      /* Start logging to syslog and stderr */
      openlog("vtund", LOG_PID | LOG_NDELAY | LOG_PERROR, LOG_DAEMON);
 
-     while( (opt=getopt(argc,argv,"isf:P:t:np")) != EOF ){
+     while( (opt=getopt(argc,argv,"misf:P:L:t:np")) != EOF ){
 	switch(opt){
+	    case 'm':
+	        if (mlockall(MCL_CURRENT | MCL_FUTURE) < 0) {
+		    perror("Unable to mlockall()");
+		    exit(-1);
+	        }
+		break;
 	    case 'i':
 		vtun.svr_type = VTUN_INETD;
 	    case 's':
 		svr = 1;
 		break;
+	    case 'L':
+		vtun.svr_addr = strdup(optarg);
+		break;
 	    case 'P':
-		vtun.svr_port = atoi(optarg);
+		vtun.bind_addr.port = atoi(optarg);
 		break;
 	    case 'f':
 		vtun.cfg_file = strdup(optarg);
@@ -112,6 +127,12 @@ int main(int argc, char *argv[], char *env[])
      }	
      reread_config(0);
 
+     if (vtun.syslog != LOG_DAEMON) {
+	/* Restart logging to syslog using specified facility  */
+ 	closelog();
+ 	openlog("vtund", LOG_PID|LOG_NDELAY|LOG_PERROR, vtun.syslog);
+     }
+
      if(!svr){
 	if( argc - optind < 2 ){
 	   usage();
@@ -120,7 +141,7 @@ int main(int argc, char *argv[], char *env[])
 	hst = argv[optind++];
 
         if( !(host = find_host(hst)) ){	
-	   syslog(LOG_ERR,"Host %s not found in %s", hst, vtun.cfg_file);
+	   vtun_syslog(LOG_ERR,"Host %s not found in %s", hst, vtun.cfg_file);
 	   exit(1);
         }
 
@@ -131,8 +152,8 @@ int main(int argc, char *argv[], char *env[])
       * Now fill uninitialized fields of the options structure
       * with default values. 
       */ 
-     if(vtun.svr_port == -1)
-	vtun.svr_port = VTUN_PORT;
+     if(vtun.bind_addr.port == -1)
+	vtun.bind_addr.port = VTUN_PORT;
      if(vtun.persist == -1)
 	vtun.persist = 0;
      if(vtun.timeout == -1)
@@ -194,7 +215,7 @@ void write_pid(void)
      FILE *f;
 
      if( !(f=fopen(VTUN_PID_FILE,"w")) ){
-        syslog(LOG_ERR,"Can't write PID file");
+        vtun_syslog(LOG_ERR,"Can't write PID file");
         return;
      }
 
@@ -205,7 +226,7 @@ void write_pid(void)
 void reread_config(int sig)
 {
      if( !read_config(vtun.cfg_file) ){
-	syslog(LOG_ERR,"No hosts defined");
+	vtun_syslog(LOG_ERR,"No hosts defined");
 	exit(1);
      }
 }
@@ -215,8 +236,9 @@ void usage(void)
      printf("VTun ver %s\n", VTUN_VER);
      printf("Usage: \n");
      printf("  Server:\n");
-     printf("\tvtund <-s> [-f file] [-P port]\n");
+     printf("\tvtund <-s> [-f file] [-P port] [-L local address]\n");
      printf("  Client:\n");
-     printf("\tvtund [-f file] [-P port] [-L local address] "
-	    "[-p] [-t timeout] <host> <server adress>\n");
+     /* I don't think these work. I'm disabling the suggestion - bish 20050601*/
+     printf("\tvtund [-f file] " /* [-P port] [-L local address] */
+	    "[-p] [-m] [-t timeout] <host profile> <server address>\n");
 }
