@@ -17,7 +17,7 @@
  */
 
 /*
- * $Id: server.c,v 1.4.2.2 2000/09/14 14:57:20 maxk Exp $
+ * server.c,v 1.4.2.5.2.4 2006/11/16 04:03:53 mtbishop Exp
  */ 
 
 #include "config.h"
@@ -50,6 +50,13 @@
 
 #include "compat.h"
 
+static volatile sig_atomic_t server_term;
+static void sig_term(int sig)
+{
+     vtun_syslog(LOG_INFO,"Terminated");
+     server_term = VTUN_SIG_TERM;
+}
+
 void connection(int sock)
 {
      struct sockaddr_in my_addr, cl_addr;
@@ -60,40 +67,42 @@ void connection(int sock)
 
      opt = sizeof(struct sockaddr_in);
      if( getpeername(sock, (struct sockaddr *) &cl_addr, &opt) ){
-        syslog(LOG_ERR, "Can't get peer name");
+        vtun_syslog(LOG_ERR, "Can't get peer name");
         exit(1);
      }
      opt = sizeof(struct sockaddr_in);
      if( getsockname(sock, (struct sockaddr *) &my_addr, &opt) < 0 ){
-        syslog(LOG_ERR, "Can't get local socket address");
+        vtun_syslog(LOG_ERR, "Can't get local socket address");
         exit(1); 
      }
 
      ip = strdup(inet_ntoa(cl_addr.sin_addr));
 
+     io_init();
+
      if( (host=auth_server(sock)) ){	
         sa.sa_handler=SIG_IGN;
 	sa.sa_flags=SA_NOCLDWAIT;;
         sigaction(SIGHUP,&sa,NULL);
 
-	syslog(LOG_INFO,"Session %s[%s:%d] opened", host->host, ip, 
+	vtun_syslog(LOG_INFO,"Session %s[%s:%d] opened", host->host, ip, 
 					ntohs(cl_addr.sin_port) );
         host->rmt_fd = sock; 
 	
         host->sopt.laddr = strdup(inet_ntoa(my_addr.sin_addr));
-        host->sopt.lport = vtun.svr_port;
+        host->sopt.lport = vtun.bind_addr.port;
         host->sopt.raddr = strdup(ip);
 	host->sopt.rport = ntohs(cl_addr.sin_port);
 
 	/* Start tunnel */
 	tunnel(host);
 
-	syslog(LOG_INFO,"Session %s closed", host->host);
+	vtun_syslog(LOG_INFO,"Session %s closed", host->host);
 
 	/* Unlock host. (locked in auth_server) */	
 	unlock_host(host);
      } else {
-        syslog(LOG_INFO,"Denied connection from %s:%d", ip,
+        vtun_syslog(LOG_INFO,"Denied connection from %s:%d", ip,
 					ntohs(cl_addr.sin_port) );
      }
      close(sock);
@@ -103,16 +112,22 @@ void connection(int sock)
 
 void listener(void)
 {
+     struct sigaction sa;
      struct sockaddr_in my_addr, cl_addr;
      int s, s1, opt;
 
      memset(&my_addr, 0, sizeof(my_addr));
      my_addr.sin_family = AF_INET;
-     my_addr.sin_addr.s_addr = INADDR_ANY;
-     my_addr.sin_port = htons(vtun.svr_port);	
+
+     /* Set listen address */
+     if( generic_addr(&my_addr, &vtun.bind_addr) < 0)
+     {
+        vtun_syslog(LOG_ERR, "Can't fill in listen socket");
+        exit(1);
+     }
 
      if( (s=socket(AF_INET,SOCK_STREAM,0))== -1 ){
-	syslog(LOG_ERR,"Can't create socket");
+	vtun_syslog(LOG_ERR,"Can't create socket");
 	exit(1);
      }
 
@@ -120,28 +135,36 @@ void listener(void)
      setsockopt(s, SOL_SOCKET, SO_REUSEADDR, &opt, sizeof(opt)); 
 
      if( bind(s,(struct sockaddr *)&my_addr,sizeof(my_addr)) ){
-	syslog(LOG_ERR,"Can't bind to the socket");
+	vtun_syslog(LOG_ERR,"Can't bind to the socket");
 	exit(1);
      }
 
      if( listen(s, 10) ){
-	syslog(LOG_ERR,"Can't listen on the socket");
+	vtun_syslog(LOG_ERR,"Can't listen on the socket");
 	exit(1);
      }
 
-     set_title("waiting for connections on port %d", vtun.svr_port);
+     memset(&sa,0,sizeof(sa));
+     sa.sa_flags = SA_NOCLDWAIT;
+     sa.sa_handler=sig_term;
+     sigaction(SIGTERM,&sa,NULL);
+     sigaction(SIGINT,&sa,NULL);
+     server_term = 0;
 
-     while(1){
+     set_title("waiting for connections on port %d", vtun.bind_addr.port);
+
+     while( (!server_term) || (server_term == VTUN_SIG_HUP) ){
         opt=sizeof(cl_addr);
-	if( (s1=accept(s,(struct sockaddr *)&cl_addr,&opt)) == -1 )
+	if( (s1=accept(s,(struct sockaddr *)&cl_addr,&opt)) < 0 )
 	   continue; 
 
 	switch( fork() ){
 	   case 0:
+	      close(s);
 	      connection(s1);
 	      break;
 	   case -1:
-	      syslog(LOG_ERR, "Couldn't fork()");
+	      vtun_syslog(LOG_ERR, "Couldn't fork()");
 	   default:
 	      close(s1);
 	      break;
@@ -161,8 +184,8 @@ void server(int sock)
      sigaction(SIGPIPE,&sa,NULL);
      sigaction(SIGUSR1,&sa,NULL);
 
-     syslog(LOG_INFO,"VTUN server ver %s (%s)", VTUN_VER,
-		vtun.svr_type == VTUN_INETD ? "inetd" : "stand" );
+     vtun_syslog(LOG_INFO,"VTUN server ver %s (%s)", VTUN_VER,
+		 vtun.svr_type == VTUN_INETD ? "inetd" : "stand" );
 
      switch( vtun.svr_type ){
 	case VTUN_STAND_ALONE:
